/**
 * AsyncImageView.java
 * 
 * Created by Tom Rudick on 8/11/09.
 * Ported by Vincent Simonetti 10/18/10.
 * Copyright Microsoft 2009. All rights reserved.
 */
package bing.images.demo;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;

import net.rim.device.api.system.Application;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.component.BitmapField;

//Basic async image field, was copied (then modified) from bing.common.BingAsyncRequest
public class AsyncImageView extends BitmapField implements Runnable
{
	private ByteArrayOutputStream responseData;
	private HttpConnection con;
	private Application app; //Little hack so everything is done under the correct app lock.
	
	public AsyncImageView()
	{
		super(null, BitmapField.FIELD_HCENTER);
	}
	
	public void loadImage(String url, Application app)
	{
		try
		{
			this.con = (HttpConnection)Connector.open(url, Connector.READ);
		}
		catch (IOException e)
		{
		}
		this.app = app;
		new Thread(this).start();
	}
	
	public void run()
	{
		int count = 0;
		InputStream in = null;
		int dataBufLen;
		byte[] dataBuf = new byte[dataBufLen = 1024];
		byte[] read;
		while(con != null)
		{
			try
			{
				switch(count)
				{
					case 0:
						//Wait for response
						int code = this.con.getResponseCode();
						if(code == HttpConnection.HTTP_OK)
						{
							//If a connection has been made then get the data length and start the ball rolling
							code = (int)this.con.getLength();
							onResponse(code);
							count++;
						}
						break;
					case 1:
						//Read the data
						if(in == null)
						{
							//If no input connection exists then open one, this will get closed then con.close is called.
							in = this.con.openInputStream();
						}
						//Get the number of currently available
						int av = in.available();
						//New data exists
						if(av > dataBufLen)
						{
							//The new data is larger then the buffer
							dataBuf = new byte[dataBufLen = av];
						}
						//Read in the data
						av = in.read(dataBuf);
						if(av > 0)
						{
							//Data has been read
							read = new byte[av];
							System.arraycopy(dataBuf, 0, read, 0, av);
							onData(read);
						}
						else if(av < 0)
						{
							//No more data exists, next step
							count++;
						}
						break;
					case 2:
						//Finish and parse
						onFinish();
						break;
				}
			}
			catch(Exception e)
			{
				//Error
				onError(e);
			}
		}
	}
	
	private synchronized void onResponse(int dataSize)
	{
		dataSize = dataSize <= 0 ? 100 : dataSize; //Just in case the server doesn't support size.
		if(this.responseData != null)
		{
			try
			{
				this.responseData.close();
			}
			catch(Exception e)
			{
			}
			this.responseData = null;
		}
		this.responseData = new ByteArrayOutputStream(dataSize);
	}
	
	private synchronized void onData(byte[] data)
	{
		try
		{
			this.responseData.write(data);
		}
		catch (Exception e)
		{
		}
	}
	
	private synchronized void onError(Exception e)
	{
		try
		{
			this.con.close();
			this.con = null;
		}
		catch(Exception ee)
		{
		}
		
		synchronized(this.app.getAppEventLock())
		{
			this.setBitmap(Bitmap.getPredefinedBitmap(Bitmap.EXCLAMATION));
		}
	}
	
	private synchronized void onFinish()
	{
		synchronized(this.app.getAppEventLock())
		{
			this.setBitmap(Bitmap.createBitmapFromBytes(this.responseData.toByteArray(), 0, -1, 1));
		}
		
		try
		{
			this.con.close();
			this.con = null;
			this.responseData.close();
			this.responseData = null;
		}
		catch(Exception ee)
		{
		}
	}
}
