/**
 * ScreenManager.java
 * 
 * Ported by Vincent Simonetti 10/18/10.
 * Copyright Microsoft 2009. All rights reserved.
 */
package bing.images.demo;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.container.HorizontalFieldManager;

//Simple manager class that displays the Bing logo when no fields are present
public class ScreenManager extends HorizontalFieldManager 
{
	private boolean noImages;
	private Bitmap logo;
	
	public ScreenManager()
	{
		this(Manager.HORIZONTAL_SCROLL);
	}
	
	public ScreenManager(long style)
	{
		super(style);
		noImages = true;
		logo = Bitmap.getBitmapResource("Logo_63x23_White.png");
	}
    
    protected void sublayout(int width, int height) 
    {
    	if(noImages)
    	{
    		setExtent(Math.min(width, Display.getWidth()), Math.min(height, Display.getHeight()));
    	}
    	else
    	{
    		super.sublayout(width, height);
    	}
    }
    
    protected void paint(Graphics graphics)
	{
		if(noImages)
		{
			graphics.drawBitmap((this.getWidth() >> 1) - (logo.getWidth() >> 1), 
					(this.getHeight() >> 1) - (logo.getHeight() >> 1), logo.getWidth(), logo.getHeight(), logo, 0, 0);
		}
		else
		{
			super.paint(graphics);
		}
	}
    
    public int getFieldAtLocation(int x, int y)
    {
    	if(noImages)
    	{
    		return -1;
    	}
    	return super.getFieldAtLocation(x, y);
    }
	
	public void add(Field field)
	{
		noImages = false;
		super.add(field);
	}
	
	public void deleteAll()
	{
		noImages = true;
		super.deleteAll();
	}
}
