//#preprocessor

/**
 * SearchScreen.java
 * 
 * Created by Tom Rudick on 8/10/09.
 * Ported by Vincent Simonetti 10/18/10.
 * Copyright Microsoft 2009. All rights reserved.
 */
package bing.images.demo;

import bing.Bing;
import bing.BingAsyncRequestNotification;
import bing.blackberry.unifiedsearch.SearchCallback;
import bing.blackberry.unifiedsearch.SearchProviderOptions;
import bing.blackberry.unifiedsearch.UnifiedSearch;
import bing.requests.BingImageRequest;
import bing.requests.BingRequest;
import bing.responses.BingImageResponse;
import bing.responses.BingResponse;
import bing.results.BingImageResult;
import bing.results.BingResult;
import net.rim.device.api.system.Application;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Manager;
//#ifndef BlackBerrySDK4.5.0 | BlackBerrySDK4.6.0 | BlackBerrySDK4.6.1
import net.rim.device.api.ui.TouchEvent;
//#endif
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
//#ifndef BlackBerrySDK4.5.0
import net.rim.device.api.ui.decor.BackgroundFactory;
//#else
import net.rim.device.api.ui.Graphics;
//#endif

public class SearchScreen extends MainScreen implements FieldChangeListener, BingAsyncRequestNotification
{
	private Manager scrollView;
	private BasicEditField editField;
	private String imageFilter;
	private SearchProviderOptions searchID;
	
	private static final String BING_ID = "Your AppID HERE";
	
	public SearchScreen()
	{
		super(NO_VERTICAL_SCROLL | USE_ALL_HEIGHT);
		
//#ifndef BlackBerrySDK4.5.0
		//Set background
		this.setBackground(BackgroundFactory.createSolidBackground(Color.BLACK));
//#endif
		
		//Create a search bar
		HorizontalFieldManager horz = new HorizontalFieldManager();
		setBanner(horz);
		
		//Search field
		horz.add(editField = new BasicEditField(BasicEditField.FOCUSABLE | BasicEditField.EDITABLE | BasicEditField.FIELD_LEFT)
		{
			protected void layout(int width, int height)
			{
				//For some reason the HorizontalFieldManager does not arrange the BasicEditField next to the BitmapField... make sure that the EditField doesn't take up all the space
				if(width == Display.getWidth())
				{
					width -= (24 + (2 * 5));
				}
				super.layout(width, height);
			}
			
//#ifdef BlackBerrySDK4.5.0
			protected void paintBackground(Graphics graphics)
			{
				int oldCol = graphics.getBackgroundColor();
				graphics.setBackgroundColor(Color.WHITE);
				graphics.clear();
				graphics.setBackgroundColor(oldCol);
				super.paintBackground(graphics);
			}
//#endif
		});
//#ifndef BlackBerrySDK4.5.0
		editField.setBackground(BackgroundFactory.createSolidBackground(Color.WHITE));
//#endif
		//Search button
		BitmapField searchButton = new BitmapField(Bitmap.getBitmapResource("Spyglass_24x24.gif"), BitmapField.FOCUSABLE | BitmapField.FIELD_RIGHT)
		{
			protected boolean navigationClick(int status, int time)
			{
				this.fieldChangeNotify(0);
				return super.navigationClick(status, time);
			}
			
//#ifndef BlackBerrySDK4.5.0 | BlackBerrySDK4.6.0 | BlackBerrySDK4.6.1
			protected boolean touchEvent(TouchEvent message)
			{
				if(message.getEvent() == TouchEvent.CLICK)
				{
					this.fieldChangeNotify(0);
					return true;
				}
				return super.touchEvent(message);
			}
//#endif
		};
		searchButton.setChangeListener(this);
		searchButton.setPadding(5, 5, 5, 5);
		horz.add(searchButton);
		
		//Setup the scroll-view and background
//#ifndef BlackBerrySDK4.5.0
		add(scrollView = new ScreenManager());
		this.getMainManager().setBackground(this.getBackground());
		scrollView.setBackground(this.getBackground());
//#else
		add(scrollView = new ScreenManager()
		{
			protected void paintBackground(Graphics graphics)
			{
				int oldCol = graphics.getBackgroundColor();
				graphics.setBackgroundColor(Color.BLACK);
				graphics.clear();
				graphics.setBackgroundColor(oldCol);
				super.paintBackground(graphics);
			}
		});
//#endif
		
		//Setup the image filter so it only searches for images that are as wide as the screen
		imageFilter = "Size:Width:" + Display.getWidth();
		
		if(UnifiedSearch.isSupported())
		{
			addSearchCallback();
		}
	}
	
//#ifdef BlackBerrySDK4.5.0
	protected void paintBackground(Graphics graphics)
	{
		int oldCol = graphics.getBackgroundColor();
		graphics.setBackgroundColor(Color.BLACK);
		graphics.clear();
		graphics.setBackgroundColor(oldCol);
		super.paintBackground(graphics);
	}
//#endif
	
	private void addSearchCallback()
	{
		if(!UnifiedSearch.registerSearchProvider(this.searchID = new SearchProviderOptions(new SearchCallback()
		{
			public void receiveBingResponse(BingResponse response)
			{
				Application app = SearchScreen.this.getApplication();
				if(!app.isForeground()/* && !app.isForegroundRequestInProgress()*/)
				{
					app.requestForeground();
				}
				SearchScreen.this.receiveBingResponse(response, app);
			}
			
			public boolean updated()
			{
				return false;
			}
			
			public boolean supportedSearch(String keywords)
			{
				return true;
			}
			
			public void searchStarted()
			{
				//SearchScreen.this.getApplication().requestForeground();
			}
			
			public BingRequest getRequest()
			{
				BingImageRequest request = new BingImageRequest();
				
				request.setFilters(imageFilter);
				
				return request;
			}
			
			public String getName()
			{
				return "Bing Image Search";
			}
			
			public Object getImage()
			{
				return Bitmap.getBitmapResource("Logo_63x23_White.png");
			}
			
			public long getContentType()
			{
				return 0x20000; //SearchableContentTypeConstants.CONTENT_TYPE_WEB;
			}
			
			public Bing getBing()
			{
				return new Bing(BING_ID);
			}
		})))
		{
			this.searchID = null;
		}
	}
	
	private void removeSearchCallback()
	{
		if(this.searchID != null)
		{
			UnifiedSearch.unregisterSearchProvider(this.searchID);
		}
	}
	
	public void fieldChanged(Field field, int context)
	{
		//Do a search for the specified images
		Bing bing = new Bing(BING_ID);
		
		BingImageRequest request = new BingImageRequest();
		
		request.setFilters(imageFilter);
		
		bing.search(editField.getText(), request, this);
	}
	
	public void receiveBingResponse(BingResponse response)
	{
		receiveBingResponse(response, null);
	}
	
	private void receiveBingResponse(BingResponse response, Application app)
	{
		if(response == null)
		{
			return;
		}
		
		if(app == null)
		{
			app = this.getApplication();
		}
		
		//Explicit app lock is desired because since all image processing is done async if invoke/AndWait is used then it is possible to have multiple images of different searches
		synchronized(app.getAppEventLock())
		{
			//Clean up the scrollView
			scrollView.deleteAll();
		}
		
		BingImageResponse iresponse = (BingImageResponse)response;
		
		BingResult[] results = iresponse.results();
		int numResults = results.length;
		
		//Take the results and open (and add) a image that will load async
		for(int i = 0; i < numResults; i++)
		{
			AsyncImageView imgView = new AsyncImageView();
			imgView.loadImage(((BingImageResult)results[i]).getMediaUrl(), app);
			
			synchronized(app.getAppEventLock())
			{
				scrollView.add(imgView);
			}
		}
	}
	
	public void close()
	{
		if(UnifiedSearch.isSupported())
		{
			removeSearchCallback();
		}
		super.close(); //Makes sure that the screen simply closes
	}
	
	public boolean onMenu(int instance)
	{
		return false; //Prevent annoying popup menus
	}
	
	protected boolean onSavePrompt()
	{
		return true; //Keep the save prompt from opening
	}
}
